@@ -393,7 +393,7 @@ err_undump:
 }
 
 int vps_restore(vps_handler *h, envid_t veid, vps_param *vps_p, int cmd,
-	cpt_param *param)
+	cpt_param *param, skipFlags skip)
 {
 	int ret, rst_fd;
 	int dump_fd = -1;
@@ -437,7 +437,7 @@ int vps_restore(vps_handler *h, envid_t veid, vps_param *vps_p, int cmd,
 	}
 	param->rst_fd = rst_fd;
 	param->cmd = cmd;
-	ret = vps_start_custom(h, veid, vps_p, SKIP_CONFIGURE,
+	ret = vps_start_custom(h, veid, vps_p, SKIP_CONFIGURE | skip,
 		NULL, restore_fn, param);
 	if (ret)
 		goto err;
