@@ -31,7 +31,7 @@
 
 #include "list.h"
 #include "logger.h"
-#include "config.h"
+#include "vzconfig.h"
 #include "vzerror.h"
 #include "util.h"
 #include "script.h"
@@ -41,8 +41,8 @@
 #include "destroy.h"
 #include "image.h"
 
-#define VPS_CREATE	LIB_SCRIPTS_DIR "vps-create"
-#define VPS_DOWNLOAD	LIB_SCRIPTS_DIR "vps-download"
+#define VPS_CREATE	SCRIPTDIR "/vps-create"
+#define VPS_DOWNLOAD	SCRIPTDIR "/vps-download"
 #define VZOSTEMPLATE	"/usr/bin/vzosname"
 
 static int vps_postcreate(envid_t veid, vps_res *res);
@@ -146,6 +146,10 @@ find:
 	}
 	dst = tmp_dir;
 	if (ploop) {
+#ifndef HAVE_PLOOP
+		ret = VZ_PLOOP_UNSUP;
+		goto err;
+#else
 		/* Create and mount ploop image */
 		struct vzctl_create_image_param param = {};
 		struct vzctl_mount_param mount_param = {};
@@ -169,6 +173,7 @@ find:
 			goto err;
 
 		dst = fs->root;
+#endif
 	}
 	if (!ploop &&
 		dq != NULL &&
@@ -194,8 +199,10 @@ find:
 	logger(0, 0, "Creating container private area (%s)", tmpl->ostmpl);
 	ret = run_script(VPS_CREATE, arg, env, 0);
 	free_arg(env);
+#ifdef HAVE_PLOOP
 	if (ploop)
 		vzctl_umount_image(tmp_dir);
+#endif
 	if (ret)
 		goto err;
 	if (quota) {
@@ -319,8 +326,7 @@ int vps_create(vps_handler *h, envid_t veid, vps_param *vps_p, vps_param *cmd_p,
 		goto err_cfg;
 	}
 	if (vps_p->opt.layout == VE_LAYOUT_PLOOP && !is_ploop_supported()) {
-		logger(-1, 0, "No ploop support in the kernel");
-		ret = VZ_BAD_KERNEL;
+		ret = VZ_PLOOP_UNSUP;
 		goto err_cfg;
 	}
 
