@@ -275,10 +275,34 @@ err:
 	return VZ_INVALID_PARAMETER_SYNTAX;
 }
 
+/* Try to restore container from suspend if its default dump file exists.
+ * Otherwise return -1.
+ */
+static int try_restore(vps_handler *h, envid_t veid, vps_param *g_p,
+		vps_param *cmd_p)
+{
+	char buf[STR_SIZE];
+	char *dumpdir = g_p->res.cpt.dumpdir;
+
+	/* if dump file exists */
+	get_dump_file(veid, dumpdir, buf, sizeof(buf));
+	if (stat_file(buf) != 1)
+		return -1;
+
+	/* Do restore */
+	logger(0, 0, "Dump file %s exists, trying to restore from it", buf);
+	cmd_p->res.cpt.dumpdir = dumpdir;
+
+	return vps_restore(h, veid, g_p, CMD_RESTORE, &cmd_p->res.cpt,
+			SKIP_UMOUNT);
+}
+
 static int start(vps_handler *h, envid_t veid, vps_param *g_p,
 	vps_param *cmd_p)
 {
 	int ret;
+	const char *private = g_p->res.fs.private;
+	skipFlags skip = SKIP_NONE;
 
 	if (g_p->opt.start_disabled == YES &&
 		cmd_p->opt.start_force != YES)
@@ -286,13 +310,31 @@ static int start(vps_handler *h, envid_t veid, vps_param *g_p,
 		logger(-1, 0, "Container start disabled");
 		return VZ_VE_START_DISABLED;
 	}
+
+	if (check_var(private, "VE_PRIVATE is not set"))
+		return VZ_VE_PRIVATE_NOTSET;
+
+	if (stat_file(private) != 1) {
+		logger(-1, 0, "Container private area %s does not exist",
+				private);
+		return VZ_FS_NOPRVT;
+	}
+
 	if (vps_is_run(h, veid)) {
 		logger(-1, 0, "Container is already running");
 		return VZ_VE_RUNNING;
 	}
+	/* Try restore first */
+	ret = try_restore(h, veid, g_p, cmd_p);
+	if (ret == 0)
+		return ret;
+	if (ret != -1) /* We tried to restore */
+		skip |= SKIP_REMOUNT; /* Do not remount just mounted fs */
+	if (cmd_p->opt.skip_setup == YES)
+		skip |= SKIP_SETUP;
 	g_p->res.misc.wait = cmd_p->res.misc.wait;
-	ret = vps_start(h, veid, g_p,
-		cmd_p->opt.skip_setup == YES ? SKIP_SETUP : 0, &g_action);
+	ret = vps_start(h, veid, g_p, skip, &g_action);
+
 	return ret;
 }
 
@@ -636,6 +678,7 @@ static int parse_snapshot_create_opt(envid_t veid, int argc, char **argv,
 	{"name", required_argument, NULL, PARAM_SNAPSHOT_NAME},
 	{"description", required_argument, NULL, PARAM_SNAPSHOT_DESC},
 	{"skip-suspend", no_argument, NULL, PARAM_SNAPSHOT_SKIP_SUSPEND},
+	{"skip-config", no_argument, NULL, PARAM_SNAPSHOT_SKIP_CONFIG},
 	{ NULL, 0, NULL, 0 }
 	};
 
@@ -753,8 +796,9 @@ static int check_set_mode(vps_handler *h, envid_t veid, int setmode, int apply,
 		found++;
 	}
 	/* Changing mount_opts */
-	if ( new_res->fs.mount_opts && strcmp(new_res->fs.mount_opts,
-				old_res->fs.mount_opts)) {
+	if ( new_res->fs.mount_opts && (!old_res->fs.mount_opts ||
+			strcmp(new_res->fs.mount_opts,
+				old_res->fs.mount_opts))) {
 		if (loud)
 			logger(-1, 0, "Unable to change mount options "
 					"on a running container");
@@ -1156,7 +1200,7 @@ static int restore(vps_handler *h, envid_t veid, vps_param *g_p,
 	if (cmd == CMD_KILL || cmd == CMD_RESUME)
 		return cpt_cmd(h, veid, g_p->res.fs.root,
 				CMD_RESTORE, cmd, cmd_p->res.cpt.ctx);
-	return vps_restore(h, veid, g_p, cmd, &cmd_p->res.cpt);
+	return vps_restore(h, veid, g_p, cmd, &cmd_p->res.cpt, SKIP_NONE);
 }
 
 static int show_status(vps_handler *h, envid_t veid, vps_param *param)
@@ -1257,10 +1301,10 @@ int parse_action_opt(envid_t veid, act_t action, int argc, char *argv[],
 	case ACTION_CUSTOM:
 		ret = parse_custom_opt(veid, argc, argv, param, name);
 		break;
-	case ACTION_CHKPNT:
+	case ACTION_SUSPEND:
 		ret = parse_chkpnt_opt(argc, argv, param);
 		break;
-	case ACTION_RESTORE:
+	case ACTION_RESUME:
 		ret = parse_restore_opt(argc, argv, param);
 		break;
 	case ACTION_CONSOLE:
@@ -1412,10 +1456,10 @@ int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 		if (ret && action == ACTION_EXEC)
 			ret = VZ_COMMAND_EXECUTION_ERROR;
 		break;
-	case ACTION_CHKPNT:
+	case ACTION_SUSPEND:
 		ret = chkpnt(h, veid, g_p, cmd_p);
 		break;
-	case ACTION_RESTORE:
+	case ACTION_RESUME:
 		ret = restore(h, veid, g_p, cmd_p);
 		break;
 	case ACTION_RUNSCRIPT:
