@@ -346,7 +346,14 @@ static void print_bool(const char *fmt, int val)
 
 static void print_onboot(struct Cveinfo *p, int index)
 {
-	print_bool("%6s", p->onboot == YES);
+	/* ONBOOT value is NONE (0), YES (1) or NO (2) */
+	if (p->onboot)
+		return print_bool("%6s", p->onboot == YES);
+
+	if (fmt_json)
+		printf("null");
+	else
+		p_buf += snprintf(p_buf, e_buf-p_buf, "%6s", "-");
 }
 
 static void print_bootorder(struct Cveinfo *p, int index)
@@ -1604,6 +1611,10 @@ static int get_ves_cpunum()
 {
 	int i;
 
+	if (access("/proc/vz/fairsched/", F_OK))
+		/* pre-RHEL6 kernel */
+		return -1;
+
 	for (i = 0; i < n_veinfo; i++) {
 		if ((veinfo[i].hide) || (veinfo[i].status != VE_RUNNING))
 			continue;
